require 'kumo_ki'
require 'logger'
require 'yaml'

require_relative 'file_loader'

module KumoConfig
  # Environment Configuration for an infrastructure stack
  class EnvironmentConfig
    class ConfigurationError < StandardError; end

    LOGGER = Logger.new(STDOUT)

    attr_reader :app_name, :env_name

    def initialize(options, logger = LOGGER)
      @env_name = options[:env_name]
      @injected_config = options[:injected_config] || {}
      @log = logger

      if options[:config_path]
        @config_file_loader = KumoConfig::FileLoader.new(config_dir_path: options[:config_path])
      elsif options[:config_dir_path]
        @log.warn "[DEPRECATION] `:config_dir_path` is deprecated, please pass in `:config_path` instead"
        @config_file_loader = KumoConfig::FileLoader.new(config_dir_path: options[:config_dir_path])
      end

    end

    def production?
      env_name == 'production'
    end

    def development?
      !%w(production staging).include? env_name
    end

    def plain_text_secrets
      @plain_text_secrets ||= decrypt_secrets(encrypted_secrets)
    end

    def config
      # a hash of all settings that apply to this environment
      @config ||= common_config.merge(env_config).merge(@injected_config)
    end

    def get_binding
      binding
    end

    private

    def kms
      @kms ||= KumoKi::KMS.new
    end

    def decrypt_secrets(secrets)
      Hash[
        secrets.map do |name, cipher_text|
          @log.debug "Decrypting '#{name}'"
          decrypt_cipher name, cipher_text
        end
      ]
    end

    def decrypt_cipher(name, cipher_text)
      if cipher_text.start_with? '[ENC,'
        begin
          [name, kms.decrypt(cipher_text[5, cipher_text.size]).to_s]
        rescue
          @log.error "Error decrypting secret '#{name}'"
          raise
        end
      else
        [name, cipher_text]
      end
    end

    def env_config_file_name
      "#{env_name}.yml"
    end

    def env_secrets_file_name
      "#{env_name}_secrets.yml"
    end

    def encrypted_secrets
      encrypted_common_secrets.merge(encrypted_env_secrets)
    end

    def encrypted_common_secrets
      @config_file_loader.load_hash('common_secrets.yml')
    end

    def encrypted_env_secrets
      secrets = @config_file_loader.load_hash(env_secrets_file_name)

      if !secrets.empty?
        secrets
      else
        @config_file_loader.load_hash('development_secrets.yml')
      end
    end

    def common_config
      @config_file_loader.load_hash('common.yml')
    end

    def env_config
      config = @config_file_loader.load_hash(env_config_file_name)
      if !config.empty?
        config
      else
        @config_file_loader.load_hash('development.yml')
      end
    end
  end
end
